
function [bhat_data,bhat_covary]=crcs(x,y,knots,X_covary)
%This the C-RCS regression code that corresponding to the following paper
%   please cite the following paper when using this code
%
%   References: Yuankai Huo, Katherine Aboud, Hakmook Kang, Laurie E. Cutting, Bennett A. Landman, 
%   “Mapping Lifetime Brain Volumetry with Covariate-Adjusted Restricted Cubic Spline Regression from Cross-sectional Multi-site MRI”. 
%   In International Conference on Medical Image Computing and Computer Assisted Intervention (MICCAI), Athens, Greece, October 2016 
%
%   INPUT ARGUMENTS:
%   x: A vector containing the covariate values x.
%   y: A vector of length(x) that contains the response values y.
%   knots: A vector of points at which the knots are to be placed.
%
%   OPTIONAL INPUT ARGUMENT:
%   X_covary: A vector of linear effect to be covaried out. 
%   If no X_covary, the function will work as carnonical RCS
%
%   OUTPUT ARGUMENTS:
%   bhat: the estimated C-RCS  coefficients of data.
%   bhat_covary: the estimated C-RCS  coefficients of covariates.
%
%   Example
%   x = [0:0.01:2*pi]';%input x
%   linear_effect = (sin(x.*3)>0.8|sin(x.*3)<-0.6).*...
%     [ones(floor(size(x,1)/2),1);-1.*ones(ceil(size(x,1)/2),1)]; %generate a linear effect
%   noise = rand(size(x))-0.5; %generate a random noise
%   y = 2*sin(x)+noise+linear_effect; %simulated observations
%   knots = prctile(x,linspace(0,100,6)); %knots
%   X_covary = linear_effect; %covary out regressor
%   [bhat,bhat_covary]=crcs(x,y,knots,X_covary);

%Code author: Yuankai Huo, 
% EECS Department
% Vandebrilt University, USA.
%
%E-mail: yuankai.huo@vanderbilt.com
% Date: June 2. 2016.
% Version: 1.0

% *************************************************************************

%make sure x,y and X_covary are 1 colume data
if size(x,2)>size(x,1);x = x';end;
if size(y,2)>size(y,1);y = y';end;

if nargin<4
    X_covary = [];
else
    if size(X_covary,2)>size(X_covary,1);X_covary = X_covary';end;
end


n=length(y);
t = knots; % knots
K = length(t); %length of knots
C = ones(n,1); %constant one for beta0

Xj = zeros(n,length(knots)-2); %Xj defined in Eq.(13) in the paper
Xu = X_covary; %Xu defined in Eq.(15) in the paper

%derive Xj as Eq.(13) in the paper
for j = 1:(K-1)     
    if j==1
        Xj(:,1) = x;
    else
        Xj(:,j) = (x-t(j-1)).^3.*(x>t(j-1))-(x-t(K-1)).^3.*(x>t(K-1)).*(t(K)-t(j-1))./(t(K)-t(K-1)) ...
            +(x-t(K)).^3.*(x>t(K)).*(t(K-1)-t(j-1))./(t(K)-t(K-1)); 
    end
end

if isempty(Xu)
    X=[C Xj]; %the design matrix
    bhat=X\y; %obtain the coefs  %equals to glmfit(X,Y,'normal','constant','off'), regress(Y,X)
    bhat_c = [];
else
    X = [C Xj Xu];
    bhat_tmp=X\y; %obtain the coefs
    bhat = bhat_tmp(1:size([C Xj],2));
    bhat_c = bhat_tmp(size([C Xj],2)+1:end);
end


%to enforece the linearity
bhat_K = sum(bhat(3:end).*(t(1:end-2)-t(end))')./(t(end)-t(end-1));  %betaK in Eq.(14)
bhat_Kplus1 = sum(bhat(3:end).*(t(1:end-2)-t(end-1))')./(t(end-1)-t(end)); %betaK+1 in Eq.(14)
bhat_data = [bhat; bhat_K; bhat_Kplus1]; 
bhat_covary = bhat_c;


end




